/*------------------------------------------------------------
TEAM05
A
177
Target: LegionMSVC7; Platform: Win32; Comment: MS Visual C++ 7 (Legionaries)
------------------------------------------------------------*/
#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <cctype>
#include <time.h>
#include <vector>
#include <cmath>
using namespace std;

const int MAXBUFLEN = 1000;
const int WARRNING_DIST = 8;

struct Point
{
	int x, y;
	Point(int x = 0, int y = 0)
	{
		this->x = x;
		this->y = y;
	}
};

bool operator == (const Point& a, const Point& b)
{
	return a.x==b.x && a.y==b.y;
}

struct Cell
{
	int owner;
	bool isVeteran;
	bool isConnected;
	Cell() : owner(-1), isVeteran(false), isConnected(false) {}
};

class SampleBot
{
	static char* BOT_NAME;
	static int maxLegions;
	int legionsCount;
	int me, enemy, size;
	FILE* inf, *outf;

	Point city [2];

	Cell** map;
	Point target;
public:
	SampleBot(FILE* inf = NULL, FILE* outf = NULL) : map(NULL), size(0), legionsCount(maxLegions)
	{
		time_t t;
		srand((unsigned)time(&t));
		this->inf = (inf) ? inf : stdin;
		this->outf = (outf) ? outf : stdout;
	}
	~SampleBot()
	{
		if (map)
		{
			int x;
			for(x=0;x<size;++x)
				delete[] map[x];
			delete[] map;
		}
	}
	char* getName()
	{
		return BOT_NAME;
	}
	bool InputData()
	{
		char buffer[MAXBUFLEN];
		int i, x, y;
		if (2 != fscanf(inf, "%d%d", &me, &size))
			return false;
		enemy = 1 - me;
		map = new Cell* [size];
		for(i = 0; i < size; ++i)
			map[i] = new Cell[size];
		for(i = 0; i < 2; i++)
		{
			fscanf(inf,"%d%d",&city[i].x,&city[i].y);
		}
		for (y = 0; y < size; y++)
		{
			for (x = 0; x < size; x++)
			{
				fscanf(inf, "%s", buffer);
				char c1 = buffer[0];
				if (strncmp(buffer,".",1) != 0)
				{
					map[x][y].owner = buffer[1] - '0';
					map[x][y].isVeteran = (c1 == 'v' || c1 == 'V');
					map[x][y].isConnected = (c1 == 'N' || c1 == 'V');
				}
			}
		}
		return true;
	}
		
	int dist(Point& a, Point& b)
	{
		return (abs(a.x-b.x)+abs(a.y-b.y));
	}

	double dist1(Point& a, Point& b)
	{
		return sqrt(double((a.x-b.x)*(a.x-b.x))+((a.y-b.y)*(a.y-b.y)));
	}

	double fitness(Point &p)
	{
		int k;

		if (map[p.x][p.y].isVeteran)
			k=2;
		else if (map[p.x][p.y].owner==enemy)
			k=3;
		else
			k=1;

		if(!(target==city[enemy]))		//warning
			return pow(25.0 / dist1(p, target),6)*k;
		//return pow(1.2, 44.0 / (dist(p, city[enemy]) + abs(17-p.x - p.y) + 1));*/
		return (pow(25.0 / dist1(p, target),3))*k;
	}

	double getRand()
	{
		return (rand()%10000)/10000.0;
	}

	int getRankNumber(vector<Point> *ps)
	{
		double sum=0;
		vector<double> a;
		int i,j,I,J;
		for (i=0;i<ps->size();i++)
			if ((*ps)[i]==city[enemy])
				return i;

		target = Point(-1,-1);
		Point p;

		if (me==0)
		{
			for (i=0;i<ps->size();i++)
				if (((*ps)[i].x-(*ps)[i].y)==0 || ((*ps)[i].x-(*ps)[i].y)==-1)
					return i;
		}
		else
		{
			for (i=0;i<size;i++)
			{
				for (j=0;j<size;j++)
					if (dist(city[me],Point(i,j))==17 && map[i][j].owner==0)
					{
						break;
					}
				if (j<size)
					break;
			}

			if (i<size)
				target = Point(i,j);
			else
			{
				target = city[enemy];
				for (i=0;i<size;i++)
					for(j=0;j<size;j++)
						if (map[i][j].owner==enemy && dist(city[me],target)>dist(city[me],Point(i,j)))
						{
							target = Point(i,j);
						}
			}

			for (i=0;i<ps->size();i++)
			{
				p = (*ps)[i];
				if (map[p.x][p.y].owner==enemy && dist(target, p)<dist(target,city[me]))
					return i;
			}
		}

		if (target.x==-1)
			target = city[enemy];

		a.resize(ps->size());
		for (i=0;i<a.size();i++)
		{
			a[i]=fitness((*ps)[i]);	//fitness
			sum+=a[i];
		}
		sum+=1;
		double r = getRand();
		i = 0;
		while(r>-(1e-3) && i<a.size())
		{
			r-=a[i]/sum;
			if (r<0)
				return i;
			i++;
		}
		return a.size()-1;
	}
	void Move()
	{
		while (legionsCount)
		{
			vector<Point> *ps = GetAccessiblePoints();
			vector<int> f;
			f.resize(ps->size());
			
			if (ps->size())
			{
				Point p = (*ps)[getRankNumber(ps)];
				MoveTo(p.x, p.y);
			}
			else
				break;

			delete ps;
		}
	}

protected:
	vector<Point>* GetAccessiblePoints()
	{
		vector<Point> *ps = new vector<Point>();
		ps->assign(0, Point(0,0));
		int x, y;
		for(y = 0; y < size; ++y)
			for(x = 0; x < size; ++x)
				if (CanMoveTo(x, y))
					ps->push_back(Point(x,y));
		return ps;
	}
	bool CanMoveTo(int x, int y)
	{
		Cell c = map[x][y];
		if (c.owner == me) return false;
		if (c.owner == enemy && legionsCount < 2) return false;
		if (c.owner == enemy && c.isVeteran && legionsCount < 3) return false;
		return MyConnected(x - 1, y) || MyConnected(x + 1, y) || MyConnected(x, y - 1) || MyConnected(x, y + 1);
	}

	bool MyConnected(int x, int y)
	{
		if (x < 0 || x >= size || y < 0 || y >= size) 
			return false;
		return map[x][y].owner == me && map[x][y].isConnected;
	}

	static void Log(char* message)
	{
		fprintf(stderr,"%s\n", message);
	}

	void MoveTo(int x, int y)
	{
		int count;
		if (map[x][y].owner != enemy) count = 1;
		else if (map[x][y].isVeteran) count = 3;
		else count = 2;
		if (legionsCount >= count)
		{
			map[x][y].owner = me;
			map[x][y].isVeteran = count > 1;
			map[x][y].isConnected = true;
		}
		int stepD = (count < legionsCount) ? count : legionsCount;
		fprintf(outf, "%d %d %d\n", x, y, stepD);
		legionsCount -= count;
		Reconnect();
	}

	void Reconnect()
	{
		for(int y=0; y<size; y++)
			for(int x=0; x<size; x++)
				map[x][y].isConnected = false;
		DFS(city[0].x, city[0].y, 0);
		DFS(city[1].x, city[1].y, 1);
	}

	void DFS(int x, int y, int me)
	{
		if (x < 0 || x >= size) return;
		if (y < 0 || y >= size) return;
		if (map[x][y].owner != me) return;
		if (map[x][y].isConnected) return;
		map[x][y].isConnected = true;
		DFS(x, y-1, me);
		DFS(x, y+1, me);
		DFS(x-1, y, me);
		DFS(x+1, y, me);
	}
};

char* SampleBot::BOT_NAME = "SUrSU #1";
int SampleBot::maxLegions = 5;

int  main()
{
	SampleBot r(stdin, stdout);
	if (r.InputData()) 
		r.Move();
	else 
		fprintf(stdout, "%s", r.getName());
}
